﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.UltraChart.Shared.Styles;
using Infragistics.UltraChart.Resources.Appearance;
using System.Collections;
using Infragistics.UltraChart.Core;
using Infragistics.UltraChart.Core.Primitives;

namespace WinChartBugDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            this.Load += new EventHandler(HandleFormLoad);
        }

        ArrayList al = new ArrayList();

        private void HandleFormLoad(object sender, EventArgs e)
        {
            SetupChart();
            SetupSampleData();
            chDemo.DataSource = al;
            chDemo.DataBind();
        }

        private void SetupSampleData()
        {
            al.Insert(0, new ChartData("0366", 432));
            al.Insert(0, new ChartData("0901", 22470));
            al.Insert(0, new ChartData("0902", -2320));
            al.Insert(0, new ChartData("0910", 266595));
            al.Insert(0, new ChartData("0912", 3620));
            al.Insert(0, new ChartData("0936", 3950));
        }

        private void SetupChart()
        {
            // Set up the chart
            Color[] chartColors;
            chartColors = new Color[] { Color.Red, Color.Green };
            chDemo.ColorModel.ModelStyle = ColorModels.CustomLinear;
            chDemo.ColorModel.CustomPalette = chartColors;
            chDemo.BarChart.BarSpacing = 0;
            chDemo.BarChart.SeriesSpacing = 0;

            chDemo.Axis.X.Visible = false;
            chDemo.Axis.X2.Visible = false;
            chDemo.Axis.X.RangeType = AxisRangeType.Automatic;
            chDemo.Axis.X.Margin.Far.Value = 0;
            chDemo.Axis.Y.Margin.Far.Value = 0;
            chDemo.Axis.Y.Margin.Near.Value = 0;
            chDemo.Axis.Y.Extent = 35;
            chDemo.Axis.Y.Labels.Visible = false;
            chDemo.Axis.Y2.Visible = false;
            chDemo.Axis.Y.Labels.SeriesLabels.Orientation = TextOrientation.Horizontal;

            TitleAppearance taTop = chDemo.TitleTop;
            taTop.Text = "Box.Highlight Bug Demo";
            taTop.HorizontalAlign = StringAlignment.Center;
            taTop.WrapText = true;
            taTop.Orientation = TextOrientation.Horizontal;

            chDemo.FillSceneGraph += new Infragistics.UltraChart.Shared.Events.FillSceneGraphEventHandler(HandleChartFillSceneGraph);
        }

        //  The chart is designed to show changes of a value between two time periods.
        //  If the change is positive (meaning this period is greater than last period) the bar should
        //  be to the right of a central point and be colored green.
        //  If the change is negative (meaning this period is less than last period) the bar should be
        //  to te left of a central point and be colored red.
        //
        //  Because the box primitive created by Box.Highlight is 'deflated' by two pixels if the value being
        //  charted is close to zero (either negative or positive) the highlight box will have a negative width.
        //  GDI+ will not draw a Rectangle that has a negative width. This results in the incorrect color showing through.
        //
        //  If you increase the size of the Form from its as-designed size eventually the bars will be large enough to avoid
        //  trying to draw a rectangle with a negative width.
        private void HandleChartFillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            SceneGraph sg = e.SceneGraph;
            for (int ndx = 0; ndx <= sg.Count - 1; ndx++)
            {
                Primitive p = sg[ndx];
                Box bb = p as Box;
                if (bb != null)
                {
                    if (bb.Value != null)
                    {
                        decimal val = Decimal.Parse(bb.Value.ToString());
                        if (val < 0)
                        {
                            // To show the correct highlight colors swap the commented-out code
                            // on the next two lines.
                            bb.Highlight(sg, Color.Red, Color.Red);
                            //MyHighlight(sg, bb, Color.Red, Color.Red);
                        }
                        else if (val > 0)
                        {
                            // To show the correct highlight colors swap the commented-out code
                            // on the next two lines.
                            bb.Highlight(sg, Color.Green, Color.Green);
                            //MyHighlight(sg, bb, Color.Green, Color.Green);
                        }
                    }
                }
            }
        }

        //  Adapted from UltraChart\ChartCore\Core\Primitives\Box.cs
        private void MyHighlight(SceneGraph sg, Box bx, Color outlineColor, Color fillColor)
        {
            Box hb = new Box(bx.rect);
            hb.PE.Fill = fillColor;
            hb.PE.FillOpacity = fillColor.A;
            hb.PE.Stroke = outlineColor;
            hb.PE.StrokeOpacity = outlineColor.A;
            hb.Path = bx.Path;
            sg.Add(hb);

        }

        // Private class to supply demo data.
        private class ChartData
        {
            public string Legend { get; private set; }
            public decimal Value { get; private set; }

            public ChartData(string theLegend, decimal theValue)
            {
                this.Legend = theLegend;
                this.Value = theValue;
            }
        }
    }
}
